/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2006 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#pragma ident	"%Z%%M%	%I%	%E% SMI"

#pragma D option quiet

BEGIN
{
	str = "foobarbarbazbarbop";
	this->success = 0;

	c = str;
	printf("strstr(\"%s\", \"%s\") = \"%s\"\n", str, c, strstr(str, c));

	c = "baz";
	printf("strstr(\"%s\", \"%s\") = \"%s\"\n", str, c, strstr(str, c));

	c = "bar";
	printf("strstr(\"%s\", \"%s\") = \"%s\"\n", str, c, strstr(str, c));

	c = "bazbarbop";
	printf("strstr(\"%s\", \"%s\") = \"%s\"\n", str, c, strstr(str, c));

	c = "barba";
	printf("strstr(\"%s\", \"%s\") = \"%s\"\n", str, c, strstr(str, c));

	printf("strstr(\"%s\", \"%s\") = \"%s\"\n",
	    strstr(str, "baz"), strstr(str, "zba"),
	    strstr(strstr(str, "baz"), strstr(str, "zba")));

	c = "";
	printf("strstr(\"%s\", \"%s\") = \"%s\"\n", str, c, strstr(str, c));

	str = "";
	printf("strstr(\"%s\", \"%s\") = \"%s\"\n", str, c, strstr(str, c));

	str = "f";
	c = "f";
	printf("strstr(\"%s\", \"%s\") = \"%s\"\n", str, c, strstr(str, c));

	this->success = 1;
}

BEGIN
/!this->success/
{
	exit(1);
}

BEGIN
/strstr(str, "barbarf") != NULL/
{
	exit(2);
}

BEGIN
/strstr(str, "foobarbarbazbarbopp") != NULL/
{
	exit(3);
}

BEGIN
{
	exit(0);
}
